/* Virtuino Iot example: ESP8266 Receive text 
 * Supported boards: all ESP8266 boards
 * 
 * This example contains the following features
 * 1. How to receive text variables from Virtuino dashboard
 * 2. How to receive numerical variables from Virtuino dashboard
 
 * Created by Ilias Lamprou
 * Updated Mar 10 2022
 */
 
#include <ESP8266WiFi.h>
#include <WebSocketsServer.h> // Download the library WebSockets by Markus Sattler from arduino library manager (ver 2.3.5)

//--- SETTINGS ----------------------------------------
  const char* ssid = "WiFi Name";                      // WIFI network SSID
  const char* password = "WiFi Password";              // WIFI network PASSWORD
  
  WebSocketsServer webSocket = WebSocketsServer(8000); // Default Virtuino Server port = 8000 
  //IPAddress ip(192, 168, 1, 150);                    // where 150 is the desired IP Address. The first three numbers must be the same as the router IP
  //IPAddress gateway(192, 168, 1, 1);                 // set gateway to match your network. Replace with your router IP

//---- Enter all the Tags here. You have to use the same Tags on Virtuino variables
  const char* text1_tag= "V10";     // tag for text 1 value
  const char* text2_tag= "V11";     // tag for text 2 value 
  const char* number1_tag= "V12";    // tag for numerical value


//---- Enter some variables to hold the last received values
  String text1_lastValue="";
  String text2_lastValue="";
  int number1_lastValue=0;
  
  
//===================================================== sendPinStatus
// It is called every time a new client is connected.
// The void informs Virtuino app about the current pin states and variable values.

void sendPinsStatus(){
  sendValue(text1_tag, text1_lastValue);      // send the text that is stored to the variable text1_lastValue
  sendValue(text2_tag, text2_lastValue);      // send the text that is stored to the variable text2_lastValue
  sendValue(number1_tag, String(number1_lastValue));  // send the text that is stored to the variable number1_lastValue
  
  // add more here...
  }

//===================================================== onValueReceived
// It is called every time a new value received
  void onValueReceived(String tag, String value){
    Serial.println("Received: tag="+tag+ "  value="+value);
    if (tag == text1_tag) {
      text1_lastValue=value;                      // copy the new value to the variable text1_tag
      if (text1_lastValue="ENABLE_ALL"){
        digitalWrite(D5,HIGH);
        digitalWrite(D6,HIGH); 
        }
        else {
          digitalWrite(D5,LOW);
          digitalWrite(D6,LOW); 
          }

    }
    if (tag == text2_tag) {
      text2_lastValue=value;                    // copy the new value to the variable text2_tag
      // do something with the new value
    }

    if (tag == number1_tag) {
      int num = value.toInt();
      number1_lastValue=num;                                       // copy the new value to the variable num1_tag
      Serial.println("number1_lastValue="+String(number1_lastValue));
      // do something with the new value
    }

    // add more here...
    
    }

//===================================================== setup
//=====================================================
  void setup() {
      Serial.begin(115200);
      while (!Serial) delay(1);
      pinMode(D5,OUTPUT);                 // on this example the pin D5 is used as OUTPUT
      pinMode(D6,OUTPUT);                 // on this example the pin D6 is used as OUTPUT
      connectToWiFiNetwork();
      webSocket.begin();
      webSocket.onEvent(webSocketEvent);
  }

//===================================================== loop
//=====================================================
  void loop() {
      webSocket.loop();     
  
      // add your code here
    
      
      
      
  }


/*=====================================================
  ======================  UTILS =======================
  =====================================================
  You don't need to make changes to the code below
*/
//===================================================== connectToWiFiNetwork
void connectToWiFiNetwork(){
  Serial.println("Connecting to "+String(ssid));
  // If you don't want to config IP manually disable the next two lines and the disables IPAddress variables  at the top of the code
  //IPAddress subnet(255, 255, 255, 0);        // set subnet mask to match your network
  //WiFi.config(ip, gateway, subnet);          // If you don't want to config IP manually disable this line
  WiFi.mode(WIFI_STA);                         // Configure the module as station only.
  WiFi.begin(ssid, password);
   while (WiFi.status() != WL_CONNECTED) {
     delay(500);
     Serial.print(".");
    }
   Serial.println("\nWiFi connected");
   Serial.println(WiFi.localIP());              // Insert this IP on Virtuino IoT server settings
}

 
//===================================================== sendValue
// This function sends a value to a Tag. 
// The tag and the value are converted to a json message before sending 

bool sendValue(const char* tag, String value){
  String json = "{\"";
  json+=tag;
  json+="\":\"";
  json+=value;
  json+="\"}";
  Serial.println("Send: "+json);
  return webSocket.broadcastTXT(json);     // This function sends the message to all connected clients.
}

//===================================================== webSocketEvent
//This is the server handler. It receives all the messages

void webSocketEvent(uint8_t num, WStype_t type, uint8_t * payload, size_t length) {
    switch(type) {
        case WStype_DISCONNECTED:
            Serial.printf("[%u] Disconnected!\n", num);
            break;
        case WStype_CONNECTED:{ 
            IPAddress ip = webSocket.remoteIP(num);
            Serial.printf("[%u] New client connected - IP: %d.%d.%d.%d \n", num, ip[0], ip[1], ip[2], ip[3]);
            sendPinsStatus();         // send the initial pin and variable values to the connected clients
            break;
        }
        case WStype_TEXT:  // a new message received 
            Serial.printf("[%u] Received: %s\n", num, payload);
            //-- The incoming payload is a json message. The following code extracts the value from json without extra library
            String str = (char*)payload;
            int p1 = str.indexOf("{\"");
            if (p1==0) {
              int p2 = str.indexOf("\":");
              if (p2>p1) {
                String tag = str.substring(p1+2,p2);
                p1 = str.indexOf(":\"",p2);
                if (p1>0) {
                   p2 = str.lastIndexOf("\"}");  
                   if (p2>0){
                      String value = str.substring(p1+2,p2);
                      onValueReceived(tag,value);        
                    }
                  }
                }
            }
            break;
    }
}
